@@ -1,7 +1,18 @@
 package Gearman::Objects;
+use version;
+$Gearman::Objects::VERSION = qv("1.130.001");
+
 use strict;
 use warnings;
 
+=head1 NAME
+
+Gearman::Objects - a parrent class for L<Gearman::Client> and L<Gearman::Worker>
+
+=head1 METHODS
+
+=cut
+
 use constant DEFAULT_PORT => 4730;
 
 use fields qw/
@@ -25,14 +36,22 @@ sub new {
         && $self->set_job_servers(
         ref($opts{job_servers})
         ? @{ $opts{job_servers} }
-        : [$opts{job_servers}]);
+        : [$opts{job_servers}]
+        );
     $opts{debug}  && $self->debug($opts{debug});
     $opts{prefix} && $self->prefix($opts{prefix});
 
     return $self;
 } ## end sub new
 
-# getter/setter
+=head2 job_servers([$js])
+
+getter/setter
+
+C<$js> may be an array reference or scalar
+
+=cut
+
 sub job_servers {
     my ($self) = shift;
     (@_) && $self->set_job_servers(@_);
@@ -50,7 +69,8 @@ sub set_job_servers {
 
 sub canonicalize_job_servers {
     my ($self) = shift;
-    my $list = ref $_[0] ? $_[0] : [@_];    # take arrayref or array
+    # take arrayref or array
+    my $list = ref $_[0] ? $_[0] : [@_];
     foreach (@$list) {
         $_ .= ':' . Gearman::Objects::DEFAULT_PORT unless /:/;
     }
@@ -63,10 +83,17 @@ sub debug {
     return $self->{debug} || 0;
 }
 
+=head2 prefix([$prefix])
+
+getter/setter
+
+=cut
 sub prefix {
     my $self = shift;
-    return $self->{prefix} unless @_;
-    $self->{prefix} = shift;
-}
+    if (@_) {
+        $self->{prefix} = shift;
+    }
+    return $self->{prefix};
+} ## end sub prefix
 
 1;
