@@ -1,4 +1,7 @@
 package Gearman::ResponseParser;
+use version;
+$Gearman::ResponseParser::VERSION = qv("1.130.001");
+
 use strict;
 use warnings;
 
@@ -11,7 +14,7 @@ sub new {
     my $class = shift;
     my %opts  = @_;
     my $src   = delete $opts{'source'};
-    die if %opts;
+    die "unsupported arguments '@{[keys %opts]}'" if %opts;
 
     my $self = bless {
         source =>
