@@ -1,16 +1,18 @@
 package Gearman::ResponseParser::Taskset;
+use version;
+$Gearman::ResponseParser::Taskset::VERSION = qv("1.130.001");
 
 use strict;
 use warnings;
 
-use Gearman::Taskset;
 use base 'Gearman::ResponseParser';
 
-no warnings "redefine";
-
 sub new {
     my ($class, %opts) = @_;
-    my $ts   = delete $opts{taskset};
+    my $ts = delete $opts{taskset};
+    ref($ts) eq "Gearman::Taskset"
+        || die "provided taskset argument is not a Gearman::Taskset reference";
+
     my $self = $class->SUPER::new(%opts);
     $self->{_taskset} = $ts;
     return $self;
