@@ -1,36 +1,84 @@
 package Gearman::Taskset;
+use version;
+$Gearman::Taskset::VERSION = qv("1.130.001");
 
 use strict;
 use warnings;
-no warnings "redefine";
+
+use Socket;
+
+=head1 NAME
+
+Gearman::Taskset - a taskset in Gearman, from the point of view of a client
+
+=head1 SYNOPSIS
+
+    use Gearman::Client;
+    my $client = Gearman::Client->new;
+
+    # waiting on a set of tasks in parallel
+    my $ts = $client->new_task_set;
+    $ts->add_task( "add" => "1+2", {...});
+    $ts->wait();
+
+
+=head1 DESCRIPTION
+
+Gearman::Taskset is a Gearman::Client's representation of tasks queue t in Gearman
+
+=head1 METHODS
+
+=cut
 
 use fields (
-    'waiting',        # { handle => [Task, ...] }
-    'client',         # Gearman::Client
-    'need_handle',    # arrayref
 
-    'default_sock',        # default socket (non-merged requests)
-    'default_sockaddr',    # default socket's ip/port
+    # { handle => [Task, ...] }
+    'waiting',
 
-    'loaned_sock',         # { hostport => socket }
-    'cancelled',           # bool, if taskset has been cancelled mid-processing
-    'hooks',               # hookname -> coderef
+    # Gearman::Client
+    'client',
+
+    # arrayref
+    'need_handle',
+
+    # default socket (non-merged requests)
+    'default_sock',
+
+    # default socket's ip/port
+    'default_sockaddr',
+
+    # { hostport => socket }
+    'loaned_sock',
+
+    # bool, if taskset has been cancelled mid-processing
+    'cancelled',
+
+    # hookname -> coderef
+    'hooks',
 );
 
 use Carp ();
-use Gearman::Client;
 use Gearman::Util;
 use Gearman::ResponseParser::Taskset;
-use Scalar::Util ()
-    ;    # i thought about weakening taskset's client, but might be too weak.
-use Time::HiRes ();
+
+# i thought about weakening taskset's client, but might be too weak.
+use Scalar::Util ();
+use Time::HiRes  ();
+
+=head2 new($client)
+
+=cut
 
 sub new {
-    my $class = shift;
-    my Gearman::Client $client = shift;
+    my $self   = shift;
+    my $client = shift;
+    ref($client) eq "Gearman::Client"
+        || Carp::croak
+        "provided client argument is not a Gearman::Client reference";
 
-    my $self = $class;
-    $self = fields::new($class) unless ref $self;
+    unless (ref $self) {
+        $self = fields::new($self);
+    }
 
     $self->{waiting}     = {};
     $self->{need_handle} = [];
@@ -58,41 +106,60 @@ sub DESTROY {
     }
 } ## end sub DESTROY
 
+=head2 run_hook($name)
+
+run a hook callback if defined
+
+=cut
+
 sub run_hook {
     my Gearman::Taskset $self = shift;
-    my $hookname = shift || return;
+    my $name = shift;
+    ($name && $self->{hooks}->{$name}) || return;
 
-    my $hook = $self->{hooks}->{$hookname};
-    return unless $hook;
+    eval { $self->{hooks}->{$name}->(@_) };
 
-    eval { $hook->(@_) };
-
-    warn "Gearman::Taskset hook '$hookname' threw error: $@\n" if $@;
+    warn "Gearman::Taskset hook '$name' threw error: $@\n" if $@;
 } ## end sub run_hook
 
+=head2 add_hook($name)
+
+add a hook
+
+=cut
+
 sub add_hook {
     my Gearman::Taskset $self = shift;
-    my $hookname = shift || return;
+    my $name = shift || return;
 
     if (@_) {
-        $self->{hooks}->{$hookname} = shift;
+        $self->{hooks}->{$name} = shift;
     }
     else {
-        delete $self->{hooks}->{$hookname};
+        delete $self->{hooks}->{$name};
     }
 } ## end sub add_hook
 
-# this method is part of the "Taskset" interface, also implemented by
-# Gearman::Client::Async, where no tasksets make sense, so instead the
-# Gearman::Client::Async object itself is also its taskset.  (the
-# client tracks all tasks).  so don't change this, without being aware
-# of Gearman::Client::Async.  similarly, don't access $ts->{client} without
-# going via this accessor.
+=head2 client ()
+
+this method is part of the "Taskset" interface, also implemented by
+Gearman::Client::Async, where no tasksets make sense, so instead the
+Gearman::Client::Async object itself is also its taskset.  (the
+client tracks all tasks).  so don't change this, without being aware
+of Gearman::Client::Async.  similarly, don't access $ts->{client} without
+going via this accessor.
+
+=cut
+
 sub client {
     my Gearman::Taskset $ts = shift;
     return $ts->{client};
 }
 
+=head2 cancel()
+
+=cut
+
 sub cancel {
     my Gearman::Taskset $ts = shift;
 
@@ -112,6 +179,10 @@ sub cancel {
     $ts->{client}      = undef;
 } ## end sub cancel
 
+#=head2 _get_loaned_sock($hostport)
+#
+#=cut
+
 sub _get_loaned_sock {
     my Gearman::Taskset $ts = shift;
     my $hostport = shift;
@@ -124,10 +195,14 @@ sub _get_loaned_sock {
     return $ts->{loaned_sock}{$hostport} = $sock;
 } ## end sub _get_loaned_sock
 
-# event loop for reading in replies
+=head2 wait()
+
+event loop for reading in replies
+
+=cut
+
 sub wait {
     my Gearman::Taskset $ts = shift;
-
     my %opts = @_;
 
     my $timeout;
@@ -183,28 +258,24 @@ sub wait {
 
                 # TODO this should remove the fd from the list, and reassign any tasks to other jobserver, or bail.
                 # We're not in an accessible place here, so if all job servers fail we must die to prevent hanging.
-                die("Job server failure: $@");
+                Carp::croak("Job server failure: $@");
             } ## end if ($@)
         } ## end foreach my $fd (keys %watching)
 
     } ## end while (!$ts->{cancelled} ...)
 } ## end sub wait
 
-# ->add_task($func, <$scalar | $scalarref>, <$uniq | $opts_hashref>
-#      opts:
-#        -- uniq
-#        -- on_complete
-#        -- on_fail
-#        -- on_status
-#        -- retry_count
-#        -- fail_after_idle
-#        -- high_priority
-# ->add_task(Gearman::Task)
-#
+=head2 add_task(Gearman::Task)
+
+=head2 add_task($func, <$scalar | $scalarref>, <$uniq | $opts_hr>
+
+C<$opts_hr> see L<Gearman::Task>
+
+=cut
 
 sub add_task {
     my Gearman::Taskset $ts = shift;
-    my $task = Gearman::Client::_get_task_from_args(@_);
+    my $task = $ts->client()->_get_task_from_args(@_);
 
     $task->taskset($ts);
 
@@ -212,12 +283,13 @@ sub add_task {
 
     my $jssock = $task->{jssock};
 
-    return $task->fail unless ($jssock);
+    return $task->fail("undefined jssock") unless ($jssock);
 
     my $req = $task->pack_submit_packet($ts->client);
     my $len = length($req);
     my $rv  = $jssock->syswrite($req, $len);
-    die "Wrote $rv but expected to write $len" unless $rv == $len;
+    $rv ||= 0;
+    Carp::croak "Wrote $rv but expected to write $len" unless $rv == $len;
 
     push @{ $ts->{need_handle} }, $task;
     while (@{ $ts->{need_handle} }) {
@@ -226,13 +298,21 @@ sub add_task {
         if (!$rv) {
             shift @{ $ts->{need_handle} }; # ditch it, it failed.
                                            # this will resubmit it if it failed.
-            return $task->fail;
-        }
+            return $task->fail(
+                join(' ',
+                    "no rv on waiting for packet",
+                    defined($rv) ? $rv : $!)
+            );
+        } ## end if (!$rv)
     } ## end while (@{ $ts->{need_handle...}})
 
     return $task->handle;
 } ## end sub add_task
 
+#
+# _get_default_soc()
+# used in Gearman::Task->taskset only
+#
 sub _get_default_sock {
     my Gearman::Taskset $ts = shift;
     return $ts->{default_sock} if $ts->{default_sock};
@@ -252,67 +332,111 @@ sub _get_default_sock {
     return $jss;
 } ## end sub _get_default_sock
 
+#
+#  _get_hashed_sock($hv)
+#
+# only used in Gearman::Task->taskset only
+#
+# return a socket
 sub _get_hashed_sock {
     my Gearman::Taskset $ts = shift;
     my $hv = shift;
 
-    my Gearman::Client $cl = $ts->{client};
-
+    my $cl = $ts->client;
+    my $sock;
     for (my $off = 0; $off < $cl->{js_count}; $off++) {
         my $idx = ($hv + $off) % ($cl->{js_count});
-        my $sock = $ts->_get_loaned_sock($cl->{job_servers}[$idx]);
-        return $sock if $sock;
+        $sock = $ts->_get_loaned_sock($cl->{job_servers}[$idx]);
+        last;
     }
 
-    return undef;
+    return $sock;
 } ## end sub _get_hashed_sock
 
+#
+#  _wait_for_packet($sock, $timeout)
+#
 # returns boolean when given a sock to wait on.
 # otherwise, return value is undefined.
 sub _wait_for_packet {
     my Gearman::Taskset $ts = shift;
-    my $sock                = shift;    # socket to singularly read from
-    my $timeout             = shift;
+
+    # socket to singularly read from
+    my $sock    = shift;
+    my $timeout = shift;
 
     my ($res, $err);
     $res = Gearman::Util::read_res_packet($sock, \$err, $timeout);
-    return 0 unless $res;
-    return $ts->_process_packet($res, $sock);
+
+    return $res ? $ts->_process_packet($res, $sock) : 0;
 } ## end sub _wait_for_packet
 
+#
+# _is_port($sock)
+#
+# return hostport || ipport
+#
 sub _ip_port {
-    my $sock = shift;
-    return undef unless $sock;
-    my $pn = getpeername($sock) or return undef;
+    my ($self, $sock) = @_;
+    $sock || return;
+
+    my $pn = getpeername($sock);
+    $pn || return;
+
+    # look for a hostport in loaned_sock
+    my $hostport;
+    while (my ($hp, $s) = each %{ $self->{loaned_sock} }) {
+        $s || next;
+        if ($sock == $s) {
+            $hostport = $hp;
+            last;
+        }
+    } ## end while (my ($hp, $s) = each...)
+
+    # hopefully it solves client->get_status mismatch
+    $hostport && return $hostport;
+
     my ($port, $iaddr) = Socket::sockaddr_in($pn);
-    return Socket::inet_ntoa($iaddr) . ":$port";
+    return join ':', Socket::inet_ntoa($iaddr), $port;
 } ## end sub _ip_port
 
+#
+# _fail_jshandle($shandle)
+#
 # note the failure of a task given by its jobserver-specific handle
+#
 sub _fail_jshandle {
     my Gearman::Taskset $ts = shift;
     my $shandle = shift;
+    $shandle
+        or Carp::croak sprintf
+        "_fail_jshandle() called without shandle parameter";
 
     my $task_list = $ts->{waiting}{$shandle}
-        or die "Uhhhh:  got work_fail for unknown handle: $shandle\n";
+        or Carp::croak "Uhhhh:  got work_fail for unknown handle: $shandle";
 
-    my Gearman::Task $task = shift @$task_list
-        or die "Uhhhh:  task_list is empty on work_fail for handle $shandle\n";
+    my $task = shift @$task_list;
+    ($task && ref($task) eq "Gearman::Task")
+        or Carp::croak
+        "Uhhhh:  task_list is empty on work_fail for handle $shandle\n";
 
-    $task->fail;
+    $task->fail("jshandle fail");
     delete $ts->{waiting}{$shandle} unless @$task_list;
 } ## end sub _fail_jshandle
 
+#
+# _process_packet($res, $sock)
+#
 sub _process_packet {
     my Gearman::Taskset $ts = shift;
     my ($res, $sock) = @_;
 
     if ($res->{type} eq "job_created") {
-        my Gearman::Task $task = shift @{ $ts->{need_handle} }
-            or die "Um, got an unexpected job_created notification";
-
+        my $task = shift @{ $ts->{need_handle} };
+        ($task && ref($task) eq "Gearman::Task")
+            or Carp::croak "Um, got an unexpected job_created notification";
         my $shandle = ${ $res->{'blobref'} };
-        my $ipport  = _ip_port($sock);
+        my $ipport  = $ts->_ip_port($sock);
 
         # did sock become disconnected in the meantime?
         if (!$ipport) {
@@ -332,16 +456,21 @@ sub _process_packet {
         return 1;
     }
 
+    my $qr = qr/(.+?)\0/;
+
     if ($res->{type} eq "work_complete") {
-        ${ $res->{'blobref'} } =~ s/^(.+?)\0//
-            or die "Bogus work_complete from server";
+        (${ $res->{'blobref'} } =~ /^$qr/)
+            or Carp::croak "Bogus work_complete from server";
+        ${ $res->{'blobref'} } =~ s/^$qr//;
         my $shandle = $1;
 
         my $task_list = $ts->{waiting}{$shandle}
-            or die "Uhhhh:  got work_complete for unknown handle: $shandle\n";
+            or Carp::croak
+            "Uhhhh:  got work_complete for unknown handle: $shandle\n";
 
-        my Gearman::Task $task = shift @$task_list
-            or die
+        my $task = shift @$task_list;
+        ($task && ref($task) eq "Gearman::Task")
+            or Carp::croak
             "Uhhhh:  task_list is empty on work_complete for handle $shandle\n";
 
         $task->complete($res->{'blobref'});
@@ -351,14 +480,22 @@ sub _process_packet {
     } ## end if ($res->{type} eq "work_complete")
 
     if ($res->{type} eq "work_exception") {
-        ${ $res->{'blobref'} } =~ s/^(.+?)\0//
-            or die "Bogus work_exception from server";
-        my $shandle   = $1;
-        my $task_list = $ts->{waiting}{$shandle}
-            or die "Uhhhh:  got work_exception for unknown handle: $shandle\n";
 
-        my Gearman::Task $task = $task_list->[0]
-            or die
+        # ${ $res->{'blobref'} } =~ s/^(.+?)\0//
+        #     or Carp::croak "Bogus work_exception from server";
+
+        (${ $res->{'blobref'} } =~ /^$qr/)
+            or Carp::croak "Bogus work_exception from server";
+        ${ $res->{'blobref'} } =~ s/^$qr//;
+        my $shandle = $1;
+
+        my $task_list = $ts->{waiting}{$shandle}
+            or Carp::croak
+            "Uhhhh:  got work_exception for unknown handle: $shandle\n";
+
+        my $task = $task_list->[0];
+        ($task && ref($task) eq "Gearman::Task")
+            or Carp::croak
             "Uhhhh:  task_list is empty on work_exception for handle $shandle\n";
 
         $task->exception($res->{'blobref'});
@@ -370,20 +507,22 @@ sub _process_packet {
         my ($shandle, $nu, $de) = split(/\0/, ${ $res->{'blobref'} });
 
         my $task_list = $ts->{waiting}{$shandle}
-            or die "Uhhhh:  got work_status for unknown handle: $shandle\n";
+            or Carp::croak
+            "Uhhhh:  got work_status for unknown handle: $shandle\n";
 
         # FIXME: the server is (probably) sending a work_status packet for each
         # interested client, even if the clients are the same, so probably need
         # to fix the server not to do that.  just put this FIXME here for now,
         # though really it's a server issue.
-        foreach my Gearman::Task $task (@$task_list) {
+        foreach my $task (@$task_list) {
             $task->status($nu, $de);
         }
 
         return 1;
     } ## end if ($res->{type} eq "work_status")
 
-    die "Unknown/unimplemented packet type: $res->{type} [${$res->{blobref}}]";
+    Carp::croak
+        "Unknown/unimplemented packet type: $res->{type} [${$res->{blobref}}]";
 
 } ## end sub _process_packet
 
