@@ -1,11 +1,25 @@
 package Gearman::Util;
+use version;
+$Gearman::Util::VERSION = qv("1.130.001");
+
 use strict;
 use warnings;
 
+# man errno
+# Resource temporarily unavailable
+# (may be the same value as EWOULDBLOCK) (POSIX.1)
 use Errno qw(EAGAIN);
 use Time::HiRes qw();
 use IO::Handle;
 
+=head1 NAME
+
+Gearman::Util
+
+=head1 METHODS
+
+=cut
+
 sub DEBUG () {0}
 
 # I: to jobserver
@@ -60,33 +74,44 @@ while (my ($num, $ary) = each %cmd) {
     $num{ $ary->[1] } = $num;
 }
 
+=head2 cmd_name($num)
+
+B<return> cmd
+
+=cut
+
 sub cmd_name {
     my $num = shift;
     my $c   = $cmd{$num};
     return $c ? $c->[1] : undef;
 }
 
+=head2 pack_req_command($key, $arg)
+
+B<return> request string
+
+=cut
+
 sub pack_req_command {
-    my $type_arg = shift;
-    my $type = $num{$type_arg} || $type_arg;
-    die "Bogus type arg of '$type_arg'" unless $type;
-    my $arg = $_[0] || '';
-    my $len = length($arg);
-    return "\0REQ" . pack("NN", $type, $len) . $arg;
-} ## end sub pack_req_command
+    return _pack_command("REQ", @_);
+}
+
+=head2 pack_res_command($cmd, $arg)
+
+B<return> response string
+
+=cut
 
 sub pack_res_command {
-    my $type_arg = shift;
-    my $type = $num{$type_arg} || int($type_arg);
-    die "Bogus type arg of '$type_arg'" unless $type;
+    return _pack_command("RES", @_);
+}
 
-    # If they didn't pass in anything to send, make it be an empty string.
-    $_[0] = '' unless defined $_[0];
-    my $len = length($_[0]);
-    return "\0RES" . pack("NN", $type, $len) . $_[0];
-} ## end sub pack_res_command
+=heade2 read_res_packet($sock, $err_ref, $timeout)
+
+B<return> undef on closed socket or malformed packet
+
+=cut
 
-# returns undef on closed socket or malformed packet
 sub read_res_packet {
     warn " Entering read_res_packet" if DEBUG;
     my $sock       = shift;
@@ -115,7 +140,7 @@ sub read_res_packet {
 
     warn " Starting up event loop\n" if DEBUG;
 
-LOOP: while (1) {
+    while (1) {
         my $time_remaining = undef;
         if (defined $timeout) {
             warn "  We have a timeout of $timeout\n" if DEBUG;
@@ -128,33 +153,17 @@ LOOP: while (1) {
 
         warn "   Got $nfound fds back from select\n" if DEBUG;
 
-        next LOOP unless vec($rout, $fileno, 1);
+        next unless vec($rout, $fileno, 1);
 
         warn "   Entering read loop\n" if DEBUG;
 
-    READ: {
-            local $!;
-            my $rv = sysread($sock, $buf, $readlen, $offset);
-
-            unless ($rv) {
-                warn "   Read error: $!\n" if DEBUG;
-                next LOOP if $! == EAGAIN;
-            }
-
-            return $err->("read_error") unless defined $rv;
-            return $err->("eof") unless $rv;
-
-            unless ($rv >= $readlen) {
-                warn
-                    "   Partial read of $rv bytes, at offset $offset, readlen was $readlen\n"
-                    if DEBUG;
-                $offset += $rv;
-                $readlen -= $rv;
-                redo READ;
-            } ## end unless ($rv >= $readlen)
-
-            warn "   Finished reading\n" if DEBUG;
-        } ## end READ:
+        my ($ok, $err_code) = _read_sock($sock, \$buf, \$readlen, \$offset);
+        if (!defined($ok)) {
+            next;
+        }
+        elsif ($ok == 0) {
+            return $err->($err_code);
+        }
 
         if (!defined $type) {
             next unless length($buf) >= 12;
@@ -165,9 +174,16 @@ LOOP: while (1) {
             $readlen = $len - $starting;
             $offset  = $starting;
 
-            #TODO rm goto
-            no warnings 'deprecated';
-            goto READ if $readlen;
+            if ($readlen) {
+                my ($ok, $err_code)
+                    = _read_sock($sock, \$buf, \$readlen, \$offset);
+                if (!defined($ok)) {
+                    next;
+                }
+                elsif ($ok == 0) {
+                    return $err->($err_code);
+                }
+            } ## end if ($readlen)
         } ## end if (!defined $type)
 
         $type = $cmd{$type};
@@ -180,13 +196,44 @@ LOOP: while (1) {
         IO::Handle::blocking($sock, 1);
 
         return {
-            'type'    => $type->[1],
-            'len'     => $len,
-            'blobref' => \$buf,
+            type    => $type->[1],
+            len     => $len,
+            blobref => \$buf,
         };
-    } ## end LOOP: while (1)
+    } ## end while (1)
 } ## end sub read_res_packet
 
+sub _read_sock {
+    my ($sock, $buf_ref, $readlen_ref, $offset_ref) = @_;
+    local $!;
+    my $rv = sysread($sock, $$buf_ref, $$readlen_ref, $$offset_ref);
+
+    unless ($rv) {
+        warn "   Read error: $!\n" if DEBUG;
+        $! == EAGAIN && return;
+    } ## end unless ($rv)
+
+    return (0, "read_error") unless defined $rv;
+    return (0, "eof")        unless $rv;
+
+    unless ($rv >= $$readlen_ref) {
+        warn
+            "   Partial read of $rv bytes, at offset $$offset_ref, readlen was $$readlen_ref\n"
+            if DEBUG;
+        $$offset_ref += $rv;
+        $$readlen_ref -= $rv;
+
+        return _read_sock($sock, $buf_ref, $readlen_ref, $offset_ref);
+    } ## end unless ($rv >= $$readlen_ref)
+
+    warn "   Finished reading\n" if DEBUG;
+    return (1);
+} ## end sub _read_sock
+
+=head2 read_text_status($sock, $err_ref)
+
+=cut
+
 sub read_text_status {
     my $sock    = shift;
     my $err_ref = shift;
@@ -216,6 +263,10 @@ sub read_text_status {
     return @lines;
 } ## end sub read_text_status
 
+=head2 send_req($sock, $reqref)
+
+=cut
+
 sub send_req {
     my ($sock, $reqref) = @_;
     return 0 unless $sock;
@@ -226,8 +277,16 @@ sub send_req {
     return ($rv && $rv == $len) ? 1 : 0;
 } ## end sub send_req
 
-# given a file descriptor number and a timeout, wait for that descriptor to
-# become readable; returns 0 or 1 on if it did or not
+=head2 wait_for_readability($fileno, $timeout)
+
+given a file descriptor number and a timeout,
+
+wait for that descriptor to become readable
+
+B<return> 0 or 1 on if it did or not
+
+=cut
+
 sub wait_for_readability {
     my ($fileno, $timeout) = @_;
     return 0 unless $fileno && $timeout;
@@ -240,4 +299,16 @@ sub wait_for_readability {
     return $nfound ? 1 : 0;
 } ## end sub wait_for_readability
 
+#
+# _pack_command($prefix, $key, $arg)
+#
+sub _pack_command {
+    my ($prefix, $key, $arg) = @_;
+    ($key && $num{$key}) || die sprintf("Bogus type arg of '%s'", $key || '');
+
+    $arg ||= '';
+    my $len = length($arg);
+    return "\0$prefix" . pack("NN", $num{$key}, $len) . $arg;
+} ## end sub _pack_command
+
 1;
