@@ -1,66 +1,105 @@
+package Gearman::Worker;
+use version;
+$Gearman::Worker::VERSION = qv("1.130.001");
 
 use strict;
 use warnings;
 
+use base "Gearman::Objects";
+
+=head1 NAME
+
+Gearman::Worker - Worker for gearman distributed job system
+
+=head1 SYNOPSIS
+
+    use Gearman::Worker;
+    my $worker = Gearman::Worker->new;
+    $worker->job_servers('127.0.0.1');
+    $worker->register_function($funcname => $subref);
+    $worker->work while 1;
+
+=head1 DESCRIPTION
+
+I<Gearman::Worker> is a worker class for the Gearman distributed job system,
+providing a framework for receiving and serving jobs from a Gearman server.
+
+Callers instantiate a I<Gearman::Worker> object, register a list of functions
+and capabilities that they can handle, then enter an event loop, waiting
+for the server to send jobs.
+
+The worker can send a return value back to the server, which then gets
+sent back to the client that requested the job; or it can simply execute
+silently.
+
+=head1 USAGE
+
+=head2 Gearman::Worker->new(%options)
+
+Creates a new I<Gearman::Worker> object, and returns the object.
+
+If I<%options> is provided, initializes the new worker object with the
+settings in I<%options>, which can contain:
+
+=over 4
+
+=item * job_servers
+
+Calls I<job_servers> (see below) to initialize the list of job
+servers. It will be ignored if this worker is running as a child
+process of a gearman server.
+
+=item * prefix
+
+Calls I<prefix> (see below) to set the prefix / namespace.
+
+=back
+
+=head2 $client-E<gt>prefix($prefix)
+
+Sets the namespace / prefix for the function names.  This is useful
+for sharing job servers between different applications or different
+instances of the same application (different development sandboxes for
+example).
+
+The namespace is currently implemented as a simple tab separated
+concatenation of the prefix and the function name.
+
+=head1 EXAMPLES
+
+=head2 Summation
+
+This is an example worker that receives a request to sum up a list of
+integers.
+
+    use Gearman::Worker;
+    use Storable qw( thaw );
+    use List::Util qw( sum );
+    my $worker = Gearman::Worker->new;
+    $worker->job_servers('127.0.0.1');
+    $worker->register_function(sum => sub { sum @{ thaw($_[0]->arg) } });
+    $worker->work while 1;
+
+See the I<Gearman::Client> documentation for a sample client sending the
+I<sum> job.
+
+=head1 METHODS
+
+=cut
+
 #TODO: retries?
 #
 use Gearman::Util;
+use Gearman::Job;
 use Carp             ();
 use IO::Socket::INET ();
 
-# this is the object that's handed to the worker subrefs
-package Gearman::Job;
-
-use fields (
-    'func',
-    'argref',
-    'handle',
-    'jss',    # job server's socket
-);
-
-sub new {
-    my ($class, $func, $argref, $handle, $jss) = @_;
-    my $self = $class;
-    $self = fields::new($class) unless ref $self;
-
-    $self->{func}   = $func;
-    $self->{handle} = $handle;
-    $self->{argref} = $argref;
-    $self->{jss}    = $jss;
-    return $self;
-} ## end sub new
-
-# ->set_status($numerator, $denominator) : $bool_sent_to_jobserver
-sub set_status {
-    my Gearman::Job $self = shift;
-    my ($nu, $de) = @_;
-
-    my $req = Gearman::Util::pack_req_command("work_status",
-        join("\0", $self->{handle}, $nu, $de));
-    die "work_status write failed"
-        unless Gearman::Util::send_req($self->{jss}, \$req);
-    return 1;
-} ## end sub set_status
-
-sub argref {
-    my Gearman::Job $self = shift;
-    return $self->{argref};
-}
-
-sub arg {
-    my Gearman::Job $self = shift;
-    return ${ $self->{argref} };
-}
-
-sub handle {
-    my Gearman::Job $self = shift;
-    return $self->{handle};
-}
-
-package Gearman::Worker;
-use base 'Gearman::Objects';
-
-use Socket qw(IPPROTO_TCP TCP_NODELAY SOL_SOCKET PF_INET SOCK_STREAM);
+use Socket qw(
+    IPPROTO_TCP
+    TCP_NODELAY
+    SOL_SOCKET
+    PF_INET
+    SOCK_STREAM);
 
 use fields (
     'sock_cache',           # host:port -> IO::Socket::INET
@@ -125,10 +164,14 @@ sub new {
     return $self;
 } ## end sub new
 
+#
+# _get_js_sock($ipport, %opts)
+#
 sub _get_js_sock {
     my Gearman::Worker $self = shift;
     my $ipport               = shift;
     my %opts                 = @_;
+    $ipport || return;
 
     my $on_connect = delete $opts{on_connect};
 
@@ -164,12 +207,17 @@ sub _get_js_sock {
         Timeout  => 1
     );
     unless ($sock) {
+        $self->debug && warn "$@";
+
         $self->{down_since}{$ipport} ||= $now;
         $self->{last_connect_fail}{$ipport} = $now;
-        return undef;
+
+        return;
     }
+
     delete $self->{last_connect_fail}{$ipport};
     delete $self->{down_since}{$ipport};
+
     $sock->autoflush(1);
     setsockopt($sock, IPPROTO_TCP, TCP_NODELAY, pack("l", 1)) or die;
 
@@ -177,15 +225,19 @@ sub _get_js_sock {
 
     unless ($self->_on_connect($sock) && $on_connect && $on_connect->($sock)) {
         delete $self->{sock_cache}{$ipport};
-        return undef;
+        return;
     }
 
     return $sock;
 } ## end sub _get_js_sock
 
+#
+# _on_connect($sock)
+#
 # Housekeeping things to do on connection to a server. Method call
 # with one argument being the 'socket' we're going to take care of.
 # returns true on success, false on failure.
+#
 sub _on_connect {
     my ($self, $sock) = @_;
 
@@ -204,6 +256,9 @@ sub _on_connect {
     return 1;
 } ## end sub _on_connect
 
+#
+# _set_ability($sock, $ability, $timeout)
+#
 sub _set_ability {
     my Gearman::Worker $self = shift;
     my ($sock, $ability, $timeout) = @_;
@@ -219,7 +274,12 @@ sub _set_ability {
     return Gearman::Util::send_req($sock, \$req);
 } ## end sub _set_ability
 
-# tell all the jobservers that this worker can't do anything
+=head2 reset_abilities
+
+tell all the jobservers that this worker can't do anything
+
+=cut
+
 sub reset_abilities {
     my Gearman::Worker $self = shift;
     my $req = Gearman::Util::pack_req_command("reset_abilities");
@@ -236,6 +296,12 @@ sub reset_abilities {
     $self->{timeouts} = {};
 } ## end sub reset_abilities
 
+=head2 uncache_sock($ipport, $reason)
+
+close TCP connection
+
+=cut
+
 sub uncache_sock {
     my ($self, $ipport, $reason) = @_;
 
@@ -248,7 +314,13 @@ sub uncache_sock {
     delete $self->{sock_cache}{$ipport};
 } ## end sub uncache_sock
 
-# does one job and returns.  no return value.
+=head2 work(%opts)
+
+Endless loop takes a job and wait for the next one.
+You can pass "stop_if", "on_start", "on_complete" and "on_fail" callbacks in I<%opts>.
+
+=cut
+
 sub work {
     my Gearman::Worker $self = shift;
     my %opts = @_;
@@ -263,6 +335,10 @@ sub work {
 
     my $last_job_time;
 
+    my $on_connect = sub {
+      return Gearman::Util::send_req($_[0], \$presleep_req);
+    };
+
     # "Active" job servers are servers that have woken us up and should be
     # queried to see if they have jobs for us to handle. On our first pass
     # in the loop we contact all servers.
@@ -287,7 +363,7 @@ sub work {
         for (my $i = 0; $i < $js_count; $i++) {
             my $js_index = ($i + $js_offset) % $js_count;
             my $js       = $jobby_js[$js_index];
-            my $jss      = $self->_get_js_sock($js)
+            my $jss      = $self->_get_js_sock($js, on_connect => $on_connect)
                 or next;
 
             # TODO: add an optional sleep in here for the test suite
@@ -397,10 +473,6 @@ sub work {
 
         my @jss;
 
-        my $on_connect = sub {
-            return Gearman::Util::send_req($_[0], \$presleep_req);
-        };
-
         foreach my $js (@{ $self->{job_servers} }) {
             my $jss = $self->_get_js_sock($js, on_connect => $on_connect)
                 or next;
@@ -443,6 +515,28 @@ sub work {
 
 } ## end sub work
 
+=head2 $worker->register_function($funcname, $subref)
+
+=head2 $worker->register_function($funcname, $timeout, $subref)
+
+Registers the function I<$funcname> as being provided by the worker
+I<$worker>, and advertises these capabilities to all of the job servers
+defined in this worker.
+
+I<$subref> must be a subroutine reference that will be invoked when the
+worker receives a request for this function. It will be passed a
+I<Gearman::Job> object representing the job that has been received by the
+worker.
+
+I<$timeout> is an optional parameter specifying how long the jobserver will
+wait for your subroutine to give an answer. Exceeding this time will result
+in the jobserver reassigning the task and ignoring your result. This prevents
+a gimpy worker from ruining the 'user experience' in many situations.
+
+The subroutine reference can return a return value, which will be sent back
+to the job server.
+=cut
+
 sub register_function {
     my Gearman::Worker $self = shift;
     my $func = shift;
@@ -466,6 +560,10 @@ sub register_function {
     $self->{can}{$ability} = $subref;
 } ## end sub register_function
 
+=head2 unregister_function($funcname)
+
+=cut
+
 sub unregister_function {
     my Gearman::Worker $self = shift;
     my $func = shift;
@@ -479,6 +577,9 @@ sub unregister_function {
     delete $self->{can}{$ability};
 } ## end sub unregister_function
 
+#
+# _register_all($req)
+#
 sub _register_all {
     my Gearman::Worker $self = shift;
     my $req = shift;
@@ -493,7 +594,16 @@ sub _register_all {
     } ## end foreach my $js (@{ $self->{...}})
 } ## end sub _register_all
 
-# getters/setters
+=head2 job_servers(@servers)
+
+override L<Gearman::Objects> method to skip job server initialization
+if defined C<$ENV{GEARMAN_WORKER_USE_STDIO}>
+
+Calling this method will do nothing in a worker that is running as a child
+process of a gearman server.
+
+=cut
+
 sub job_servers {
     my Gearman::Worker $self = shift;
     return if ($ENV{GEARMAN_WORKER_USE_STDIO});
@@ -504,113 +614,6 @@ sub job_servers {
 1;
 __END__
 
-=head1 NAME
-
-Gearman::Worker - Worker for gearman distributed job system
-
-=head1 SYNOPSIS
-
-    use Gearman::Worker;
-    my $worker = Gearman::Worker->new;
-    $worker->job_servers('127.0.0.1');
-    $worker->register_function($funcname => $subref);
-    $worker->work while 1;
-
-=head1 DESCRIPTION
-
-I<Gearman::Worker> is a worker class for the Gearman distributed job system,
-providing a framework for receiving and serving jobs from a Gearman server.
-
-Callers instantiate a I<Gearman::Worker> object, register a list of functions
-and capabilities that they can handle, then enter an event loop, waiting
-for the server to send jobs.
-
-The worker can send a return value back to the server, which then gets
-sent back to the client that requested the job; or it can simply execute
-silently.
-
-=head1 USAGE
-
-=head2 Gearman::Worker->new(%options)
-
-Creates a new I<Gearman::Worker> object, and returns the object.
-
-If I<%options> is provided, initializes the new worker object with the
-settings in I<%options>, which can contain:
-
-=over 4
-
-=item * job_servers
-
-Calls I<job_servers> (see below) to initialize the list of job
-servers. It will be ignored if this worker is running as a child
-process of a gearman server.
-
-=item * prefix
-
-Calls I<prefix> (see below) to set the prefix / namespace.
-
-=back
-
-=head2 $worker->job_servers(@servers)
-
-Initializes the worker I<$worker> with the list of job servers in I<@servers>.
-I<@servers> should contain a list of IP addresses, with optional port numbers.
-For example:
-
-    $worker->job_servers('127.0.0.1', '192.168.1.100:4730');
-
-If the port number is not provided, 4730 is used as the default.
-
-Calling this method will do nothing in a worker that is running as a child
-process of a gearman server.
-
-=head2 $worker->register_function($funcname, $subref)
-
-=head2 $worker->register_function($funcname, $timeout, $subref)
-
-Registers the function I<$funcname> as being provided by the worker
-I<$worker>, and advertises these capabilities to all of the job servers
-defined in this worker.
-
-I<$subref> must be a subroutine reference that will be invoked when the
-worker receives a request for this function. It will be passed a
-I<Gearman::Job> object representing the job that has been received by the
-worker.
-
-I<$timeout> is an optional parameter specifying how long the jobserver will
-wait for your subroutine to give an answer. Exceeding this time will result
-in the jobserver reassigning the task and ignoring your result. This prevents
-a gimpy worker from ruining the 'user experience' in many situations.
-
-The subroutine reference can return a return value, which will be sent back
-to the job server.
-
-=head2 $client-E<gt>prefix($prefix)
-
-Sets the namespace / prefix for the function names.  This is useful
-for sharing job servers between different applications or different
-instances of the same application (different development sandboxes for
-example).
-
-The namespace is currently implemented as a simple tab separated
-concatenation of the prefix and the function name.
-
-=head2 Gearman::Job->arg
-
-Returns the scalar argument that the client sent to the job server.
-
-=head2 Gearman::Job->set_status($numerator, $denominator)
-
-Updates the status of the job (most likely, a long-running job) and sends
-it back to the job server. I<$numerator> and I<$denominator> should
-represent the percentage completion of the job.
-
-=head2 Gearman::Job->work(%opts)
-
-Do one job and returns (no value returned).
-You can pass "on_start" "on_complete" and "on_fail" callbacks in I<%opts>.
-
 =head1 WORKERS AS CHILD PROCESSES
 
 Gearman workers can be run as child processes of a parent process
@@ -621,22 +624,3 @@ variable is set to true, then the jobservers function and option for
 new() are ignored and the unix socket bound to STDIN/OUT are used
 instead as the IO path to the gearman server.
 
-=head1 EXAMPLES
-
-=head2 Summation
-
-This is an example worker that receives a request to sum up a list of
-integers.
-
-    use Gearman::Worker;
-    use Storable qw( thaw );
-    use List::Util qw( sum );
-    my $worker = Gearman::Worker->new;
-    $worker->job_servers('127.0.0.1');
-    $worker->register_function(sum => sub { sum @{ thaw($_[0]->arg) } });
-    $worker->work while 1;
-
-See the I<Gearman::Client> documentation for a sample client sending the
-I<sum> job.
-
-=cut
